SystemOrganization addCategory: #'Zinc-Resource-Meta-FileSystem'!

!AbstractFileReference methodsFor: '*zinc-resource-meta-filesystem' stamp: 'SvenVanCaekenberghe 1/14/2013 10:01'!
asZnUrl
	"Convert the receiver in a file:// ZnUrl.
	Only an absolute path can be represented as a file:// URL"
	
	^ self asAbsolute path asZnUrl! !

Object subclass: #ZnFileRetrieveContentsUrlOperation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-FileSystem'!

!ZnFileRetrieveContentsUrlOperation commentStamp: '<historical>' prior: 0!
I am ZnFileRetrieveContentsUrlOperation.

I implement the #retrieveContents URL operation for FILE urls.

  ZnFileRetrieveContentsUrlOperation initialize.

Part of Zinc HTTP Components.
!

!ZnFileRetrieveContentsUrlOperation class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 10/30/2013 15:38'!
initialize
	"Register a handler named #fileRetrieveContents 
	that implements #retrieveContents for File urls"
	
	ZnUrl 
		registerOperation: #fileRetrieveContents 
		when: [ :url :operation | 
			operation = #retrieveContents and: [ url scheme = #file ] ] 
		do: [ :url :operation |
			self retrieveContentsOf: url ]! !

!ZnFileRetrieveContentsUrlOperation class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 10/30/2013 15:42'!
retrieveContentsOf: url
	"Download and return the FILE resource that url refers to.
	This will typically return a String or a ByteArray (see AbstractFileReference>>#contents).
	If something goes wrong, an exception will be signalled."
	
	^ url asFileReference contents! !

!ZnUrl methodsFor: '*zinc-resource-meta-filesystem' stamp: 'SvenVanCaekenberghe 1/14/2013 11:37'!
asFileReference
	"Convert the receiver into a new FileReference object.
	Note that for a FileReference a trailing slash is not relevant"

	| path |
	self assert: self scheme = #file description: 'Only a file:// URL can be converted to a FileReference'.
	self isSlash
		ifTrue: [ ^ FileSystem root ].
	path := self isDirectoryPath
		ifTrue: [ segments allButLast ]
		ifFalse: [ segments copy ].
	^ FileReference fileSystem: FileSystem disk path: (AbsolutePath withAll: path)! !

!AbsolutePath methodsFor: '*zinc-resource-meta-filesystem' stamp: 'SvenVanCaekenberghe 1/14/2013 10:03'!
asZnUrl
	"Convert the receiver in a file:// ZnUrl"

	| fileUrl |
	fileUrl := ZnUrl new.
	fileUrl scheme: #file.
	self do: [ :each | fileUrl addPathSegment: each ].
	^ fileUrl! !

TestCase subclass: #ZnFileUrlTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-FileSystem'!

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/6/2013 04:56'!
testAsFileReference
	| fileUrl fileReference |
	fileUrl := 'file:///foo/bar/test.txt' asZnUrl.
	fileReference := fileUrl asFileReference.
	self assert: fileReference basename equals: 'test.txt'.
	self assert: fileReference pathSegments asArray equals: #( 'foo' 'bar' 'test.txt' ).
	self assert: fileReference isAbsolute ! !

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/6/2013 04:54'!
testAsZnUrl
	| fileReference fileUrl |
	fileReference := FileLocator root / 'foo' / 'bar' / 'test.txt'.
	fileUrl := fileReference asZnUrl.
	self assert: fileUrl isFile.
	self assert: fileUrl directory equals: 'foo/bar'.
	self assert: fileUrl file equals: 'test.txt'.
	self assert: fileUrl pathSegments asArray equals: #( 'foo' 'bar' 'test.txt').
	self assert: fileUrl pathPrintString equals: '/foo/bar/test.txt'.
	self assert: fileUrl printString equals: 'file:///foo/bar/test.txt'.
	self deny: fileUrl hasHost.
	self deny: fileUrl hasPort.
	self deny: fileUrl hasQuery.
	self deny: fileUrl hasFragment! !

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 12:30'!
testRelative
	| fileReference fileUrl |
	fileReference := 'foo/bar/test.txt' asFileReference.
	self assert: fileReference isRelative.
	fileUrl := fileReference asZnUrl.
	self assert: fileUrl isFile.
	self assert: fileUrl pathSegments size >= fileReference pathSegments size.
	self assert: (fileUrl pathSegments asArray endsWith: fileReference pathSegments asArray)! !

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/30/2013 16:03'!
testRetrieveContents
	| fileReference contents url |
	fileReference := ('file-{1}.txt' format: {(1000 atRandom)}) asFileReference.
	contents := 'Testing {1} and counting.{2}' format: {(1000 atRandom). (String crlf)}.
	fileReference ensureDelete.
	fileReference writeStreamDo: [ :stream | stream nextPutAll: contents ].
	url := fileReference asZnUrl.
	self assert: url isFile.
	self assert: url retrieveContents equals: contents.
	fileReference ensureDelete! !

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/6/2013 04:51'!
testRoot
	| fileUrl fileReference |
	fileReference := FileLocator root.
	self assert: fileReference isRoot.
	fileUrl := fileReference asZnUrl.
	self assert: fileUrl isFile.
	self assert: fileUrl isSlash! !

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 12:10'!
testSlash
	| fileUrl fileReference |
	fileUrl := 'file:///' asZnUrl.
	self assert: fileUrl isSlash.
	self assert: fileUrl isFile.
	fileReference := fileUrl asFileReference.
	self assert: fileReference isRoot! !

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/6/2013 04:53'!
testSpaces
	self 
		assert: 'file:///foo%20bar/test.st' asZnUrl asFileReference 
		equals: FileLocator root / 'foo bar' / 'test.st'.
	self 
		assert: (FileLocator root / 'foo bar' / 'test.st') asZnUrl 
		equals: 'file:///foo%20bar/test.st' asZnUrl! !

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 12:23'!
testTrailingSlash
	| fileReference1 fileReference2 fileUrl1 fileUrl2 |
	
	fileReference1 := '/foo/bar' asFileReference.
	fileReference2 := '/foo/bar/' asFileReference.
	self assert: fileReference1 equals: fileReference2.
	self assert: fileReference1 asZnUrl equals: fileReference2 asZnUrl.
	
	fileUrl1 := 'file:///foo/bar' asZnUrl.
	fileUrl2 := 'file:///foo/bar/' asZnUrl.
	self deny: fileUrl1 = fileUrl2.
	self assert: fileUrl1 asFileReference equals: fileUrl2 asFileReference! !

!ZnFileUrlTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 12:17'!
testWrongScheme
	self should: [ 'http://zn.stfx.eu/zn/small.html' asZnUrl asFileReference ] raise: AssertionFailure! !
ZnFileRetrieveContentsUrlOperation initialize!
